/**
   @author Shin'ichiro Nakaoka
*/

#include "NameServerView.h"
#include <cnoid/CorbaUtil>
#include <cnoid/ExtensionManager>
#include <cnoid/TreeWidget>
#include <cnoid/Button>
#include <cnoid/SpinBox>
#include <cnoid/LineEdit>
#include <QBoxLayout>
#include <boost/bind.hpp>
#include "gettext.h"

using namespace std;
using namespace boost;
using namespace cnoid;

namespace cnoid {

    class NameServerViewImpl
    {
    public:
        NameServerViewImpl(NameServerView* self);
        ~NameServerViewImpl();
        void updateObjectList();
        void appendBindingList(CosNaming::BindingList_var& bList);

        TreeWidget treeWidget;
        LineEdit hostAddressBox;
        SpinBox portNumberSpin;

        NamingContextHelper ncHelper;
    };
}


NameServerView* NameServerView::initialize(ExtensionManager* ext)
{
    static bool initialized = false;
    static NameServerView* nameServerView = new NameServerView();
    if(!initialized){
        ext->addView(nameServerView);
        initialized = true;
    }
    return nameServerView;
}


NameServerView::NameServerView()
{
    impl = new NameServerViewImpl(this);
}


NameServerViewImpl::NameServerViewImpl(NameServerView* self)
{
    self->setName(N_("Nameserver"));
    self->setDefaultLayoutArea(View::LEFT_BOTTOM);

    QVBoxLayout* vbox = new QVBoxLayout();
    
    QHBoxLayout* hbox = new QHBoxLayout();
    hostAddressBox.setText("localhost");
    hostAddressBox.sigEditingFinished().connect
        (bind(&NameServerViewImpl::updateObjectList, this));
    hbox->addWidget(&hostAddressBox);

    portNumberSpin.setRange(0, 65535);
    portNumberSpin.setValue(2809);
    portNumberSpin.sigEditingFinished().connect
        (bind(&NameServerViewImpl::updateObjectList, this));
    hbox->addWidget(&portNumberSpin);

    PushButton* updateButton = new PushButton(_("Update"));
    updateButton->sigClicked().connect(bind(&NameServerViewImpl::updateObjectList, this));
    hbox->addWidget(updateButton);

    vbox->addLayout(hbox);

    treeWidget.setHeaderLabel(_("Object name"));
    vbox->addWidget(&treeWidget);

    self->setLayout(vbox);
}


NameServerView::~NameServerView()
{
    delete impl;
}


NameServerViewImpl::~NameServerViewImpl()
{

}


void NameServerViewImpl::updateObjectList()
{
    treeWidget.clear();

    ncHelper.setLocation(hostAddressBox.string(), portNumberSpin.value());
    
    if(ncHelper.isAlive()){

        NamingContextHelper::ObjectInfoList objects = ncHelper.getObjectList();

        for(size_t i=0; i < objects.size(); ++i){

            const NamingContextHelper::ObjectInfo& info = objects[i];
            if(info.isAlive){

                QTreeWidgetItem* item = new QTreeWidgetItem();
                QString name = info.id.c_str();
                if(!info.kind.empty()){
                    name += QString("(%1)").arg(QString::fromStdString(info.kind));
                }
                item->setText(0, name);
                treeWidget.addTopLevelItem(item);
            }
        }
    }
}
