
#include "DynamicsSimulator_impl.h"
#include <cnoid/Link>
#include <cnoid/Sensor>


using namespace std;
using namespace cnoid;
using namespace OpenHRP;


DynamicsSimulator_impl::DynamicsSimulator_impl(BodyPtr body)
{
    this->body = body;
}


DynamicsSimulator_impl::~DynamicsSimulator_impl()
{

}


void DynamicsSimulator_impl::destroy()
{

}


#ifdef OPENHRP_3_0
void DynamicsSimulator_impl::registerCharacter(const char* name, CharacterInfo_ptr cinfo)
{

}
#elif OPENHRP_3_1
void DynamicsSimulator_impl::registerCharacter(const char* name, BodyInfo_ptr cinfo)
{


}
#endif

#ifdef OPENHRP_3_0
void DynamicsSimulator_impl::registerCollisionCheckPair(
    const char* char1, const char* name1, const char* char2, const char* name2,
    ::CORBA::Double staticFriction, ::CORBA::Double slipFriction, const DblSequence6& K, const DblSequence6& C)
{

}
#elif OPENHRP_3_1
void DynamicsSimulator_impl::registerCollisionCheckPair(
    const char* char1, const char* name1, const char* char2, const char* name2,
    ::CORBA::Double staticFriction, ::CORBA::Double slipFriction, const DblSequence6& K, const DblSequence6& C, ::CORBA::Double culling_thresh)
{

}

void DynamicsSimulator_impl::registerIntersectionCheckPair(const char* char1, const char* name1, const char* char2, const char* name2, ::CORBA::Double tolerance)
{

}
#endif


void DynamicsSimulator_impl::init(
    ::CORBA::Double timeStep, OpenHRP::DynamicsSimulator::IntegrateMethod integrateOpt, OpenHRP::DynamicsSimulator::SensorOption sensorOpt)
{

}



void DynamicsSimulator_impl::registerVirtualLink(
    const char* char1, const char* link1, const char* char2, const char* link2,
    const LinkPosition& relTransform, ::CORBA::Short transformDefined, const DblSequence9& constraint, const char* connectionName)
{

}

    
void DynamicsSimulator_impl::getConnectionConstraintForce(
    const char* characterName, const char* connectionName, DblSequence6_out contactForce)
{

}


void DynamicsSimulator_impl::getCharacterSensorValues
(const char* character, const char* sensorName, DblSequence_out out_values)
{
    DblSequence_var values = new DblSequence;

    Sensor* sensor = body->sensor<Sensor>(sensorName);

    if(sensor){
        switch(sensor->type){

        case Sensor::FORCE:
        {
            ForceSensor* forceSensor = static_cast<ForceSensor*>(sensor);
            values->length(6);
            values[0] = forceSensor->f(0);
            values[1] = forceSensor->f(1);
            values[2] = forceSensor->f(2);
            values[3] = forceSensor->tau(0);
            values[4] = forceSensor->tau(1);
            values[5] = forceSensor->tau(2);
        }
        break;

        case Sensor::RATE_GYRO:
        {
            RateGyroSensor* gyro = static_cast<RateGyroSensor*>(sensor);
            values->length(3);
            values[0] = gyro->w(0);
            values[1] = gyro->w(1);
            values[2] = gyro->w(2);
        }
        break;

        case Sensor::ACCELERATION:
        {
            AccelSensor* accelSensor = static_cast<AccelSensor*>(sensor);
            values->length(3);
            values[0] = accelSensor->dv(0);
            values[1] = accelSensor->dv(1);
            values[2] = accelSensor->dv(2);
        }
        break;

        case Sensor::RANGE:
        {
            RangeSensor *rangeSensor = static_cast<RangeSensor*>(sensor);
            values->length(rangeSensor->distances.size());
            for(unsigned int i=0; i<rangeSensor->distances.size(); i++){
                values[i] = rangeSensor->distances[i];
            }
        }
        break;

        default:
            break;
        }
    }

    out_values = values._retn();
}

    
void DynamicsSimulator_impl::initSimulation()
{

}


void DynamicsSimulator_impl::stepSimulation()
{

}


void DynamicsSimulator_impl::setCharacterLinkData(
    const char* character, const char* linkName, OpenHRP::DynamicsSimulator::LinkDataType type, const DblSequence& data)
{
    Link* link = body->link(linkName);

    if(link){
	switch(type) {

	case OpenHRP::DynamicsSimulator::POSITION_GIVEN:
            link->isHighGainMode = (data[0] > 0.0);
            break;
            
	case OpenHRP::DynamicsSimulator::JOINT_VALUE:
            if(link->jointType != Link::FIXED_JOINT)
                link->q = data[0];
            break;

	case OpenHRP::DynamicsSimulator::JOINT_VELOCITY:
            if(link->jointType != Link::FIXED_JOINT)
    		link->dq = data[0];
            break;
            
	case OpenHRP::DynamicsSimulator::JOINT_ACCELERATION:
            if(link->jointType != Link::FIXED_JOINT)
                link->ddq = data[0];
            break;
            
	case OpenHRP::DynamicsSimulator::JOINT_TORQUE:
            if(link->jointType != Link::FIXED_JOINT)
                link->u = data[0];
            break;
            
	case OpenHRP::DynamicsSimulator::ABS_TRANSFORM:
        {
            link->p = Eigen::Map<Vector3>(const_cast<double*>(&data[0]));
            Matrix3 R;
            R << data[3], data[4],  data[5],
                 data[6], data[7],  data[8],
                 data[9], data[10], data[11];
            link->setSegmentAttitude(R);
        }
        break;
	
	case OpenHRP::DynamicsSimulator::ABS_VELOCITY:
        {
            link->v = Eigen::Map<Vector3>(const_cast<double*>(&data[0]));
            link->w = Eigen::Map<Vector3>(const_cast<double*>(&data[3]));
        }
        break;
        
	case OpenHRP::DynamicsSimulator::EXTERNAL_FORCE:
        {
            link->fext   = Eigen::Map<Vector3>(const_cast<double*>(&data[0]));
            link->tauext = Eigen::Map<Vector3>(const_cast<double*>(&data[3]));
            break;
        }
	
	default:
            break;
	}
    }
}


void DynamicsSimulator_impl::getCharacterLinkData(
    const char* characterName, const char* linkName, OpenHRP::DynamicsSimulator::LinkDataType type, DblSequence_out out_rdata)
{
    Link* link = body->link(linkName);

    DblSequence_var rdata = new DblSequence;

    if(link){

	switch(type) {

	case OpenHRP::DynamicsSimulator::JOINT_VALUE:
            rdata->length(1);
            rdata[0] = link->q;
            break;

	case OpenHRP::DynamicsSimulator::JOINT_VELOCITY:
            rdata->length(1);
            rdata[0] = link->dq;
            break;
            
	case OpenHRP::DynamicsSimulator::JOINT_ACCELERATION:
            rdata->length(1);
            rdata[0] = link->ddq;
            break;
            
	case OpenHRP::DynamicsSimulator::JOINT_TORQUE:
            rdata->length(1);
            rdata[0] = link->u;
            break;
            
	case OpenHRP::DynamicsSimulator::ABS_TRANSFORM:
        {
            rdata->length(12);
            rdata[0] = link->p(0);
            rdata[1] = link->p(1);
            rdata[2] = link->p(2);

            Matrix3 R = link->segmentAttitude();
            for(int i=0; i < 3; ++i){
                for(int j=0; j < 3; ++j){
                    rdata[3 + i * 3 + j] = R(i, j);
                }
            }
        }
        break;
        
	case OpenHRP::DynamicsSimulator::ABS_VELOCITY:
            rdata->length(6);
            rdata[0] = link->v(0);
            rdata[1] = link->v(1);
            rdata[2] = link->v(2);
            rdata[3] = link->w(0);
            rdata[4] = link->w(1);
            rdata[5] = link->w(2);
            break;
            
	case OpenHRP::DynamicsSimulator::EXTERNAL_FORCE:
            rdata->length(6);
            rdata[0] = link->fext(0);
            rdata[1] = link->fext(1);
            rdata[2] = link->fext(2);
            rdata[3] = link->tauext(0);
            rdata[4] = link->tauext(1);
            rdata[5] = link->tauext(2);
            break;
            
	default:
            break;
	}
    }
        
    out_rdata = rdata._retn();
}


void DynamicsSimulator_impl::getCharacterAllLinkData(
    const char* characterName, OpenHRP::DynamicsSimulator::LinkDataType type, DblSequence_out rdata)
{
    int n = body->numJoints();
    rdata = new DblSequence();
    rdata->length(n);

    switch(type) {

    case OpenHRP::DynamicsSimulator::JOINT_VALUE:
        for(int i=0; i < n; ++i){
            (*rdata)[i] = body->joint(i)->q;
        }
        break;

    case OpenHRP::DynamicsSimulator::JOINT_VELOCITY:
        for(int i=0; i < n; ++i){
            (*rdata)[i] = body->joint(i)->dq;
        }
        break;
        
    case OpenHRP::DynamicsSimulator::JOINT_ACCELERATION:
        for(int i=0; i < n; ++i){
            (*rdata)[i] = body->joint(i)->ddq;
        }
        break;
        
    case OpenHRP::DynamicsSimulator::JOINT_TORQUE:
        for(int i=0; i < n; ++i){
            (*rdata)[i] = body->joint(i)->u;
        }
        break;
        
    default:
        // put error here
        break;
    }
}


void DynamicsSimulator_impl::setCharacterAllLinkData(
    const char* characterName, OpenHRP::DynamicsSimulator::LinkDataType type, const DblSequence& wdata)
{
    int n = wdata.length();
    if(n > body->numJoints()){
        n = body->numJoints();
    }

    switch(type) {
        
    case OpenHRP::DynamicsSimulator::JOINT_VALUE:
        for(int i=0; i < n; ++i){
            if(body->joint(i)->jointType != Link::FIXED_JOINT)
                body->joint(i)->q = wdata[i];
        }
        break;
        
    case OpenHRP::DynamicsSimulator::JOINT_VELOCITY:
        for(int i=0; i < n; ++i){
            if(body->joint(i)->jointType != Link::FIXED_JOINT)
                body->joint(i)->dq = wdata[i];
        }
        break;
        
    case OpenHRP::DynamicsSimulator::JOINT_ACCELERATION:
        for(int i=0; i < n; ++i){
            if(body->joint(i)->jointType != Link::FIXED_JOINT)
                body->joint(i)->ddq = wdata[i];
        }
        break;
        
    case OpenHRP::DynamicsSimulator::JOINT_TORQUE:
        for(int i=0; i < n; ++i){
            if(body->joint(i)->jointType != Link::FIXED_JOINT){
                body->joint(i)->u = wdata[i];
            }
        }
        break;
        
    default:
        // put error here
        break;
    }
}


void DynamicsSimulator_impl::setGVector(const DblSequence3& wdata)
{

}


void DynamicsSimulator_impl::getGVector(DblSequence3_out wdata)
{

}


void DynamicsSimulator_impl::setCharacterAllJointModes(const char* characterName, OpenHRP::DynamicsSimulator::JointDriveMode jointMode)
{

}


::CORBA::Boolean DynamicsSimulator_impl::calcCharacterInverseKinematics(
    const char* characterName, const char* baseLink, const char* targetLink, const LinkPosition& target)
{
    return true;
}


void DynamicsSimulator_impl::calcCharacterForwardKinematics(const char* characterName)
{

}


void DynamicsSimulator_impl::calcWorldForwardKinematics()
{

}


#ifdef OPENHRP_3_0
void DynamicsSimulator_impl::checkCollision()
{

}

#elif OPENHRP_3_1
::CORBA::Boolean DynamicsSimulator_impl::checkCollision(::CORBA::Boolean checkAll)
{
	return false;
}
LinkPairSequence* DynamicsSimulator_impl::checkIntersection(::CORBA::Boolean checkAll)
{
	return 0;
}
DistanceSequence* DynamicsSimulator_impl::checkDistance()
{
	return 0;
}
#endif


void DynamicsSimulator_impl::getWorldState(WorldState_out wstate)
{

}


void DynamicsSimulator_impl::getCharacterSensorState(const char* characterName, SensorState_out sstate)
{
    sstate = new SensorState;
    
    int numJoints = body->numJoints();
    sstate->q.length(numJoints);
    sstate->dq.length(numJoints);
    sstate->u.length(numJoints);
    sstate->force.length(body->numSensors(Sensor::FORCE));
    sstate->rateGyro.length(body->numSensors(Sensor::RATE_GYRO));
    sstate->accel.length(body->numSensors(Sensor::ACCELERATION));
    
    for(int j=0; j < numJoints; j++){
        Link* joint = body->joint(j);
        sstate->q [j] = joint->q;
        sstate->dq[j] = joint->dq;
        sstate->u [j] = joint->u;
    }

    int n = body->numSensors(Sensor::FORCE);
    for(int id = 0; id < n; ++id){
        ForceSensor* sensor = body->sensor<ForceSensor>(id);
        Eigen::Map<Vector3>(&sstate->force[id][0]) = sensor->f;
        Eigen::Map<Vector3>(&sstate->force[id][3]) = sensor->tau;
    }

    n = body->numSensors(Sensor::RATE_GYRO);
    for(int id=0; id < n; ++id){
        RateGyroSensor* sensor = body->sensor<RateGyroSensor>(id);
        Eigen::Map<Vector3>(sstate->rateGyro[id]) = sensor->w;
    }

    n = body->numSensors(Sensor::ACCELERATION);
    for(int id=0; id < n; ++id){
        AccelSensor* sensor = body->sensor<AccelSensor>(id);
        Eigen::Map<Vector3>(sstate->accel[id]) = sensor->dv;
    }		
}


::CORBA::Boolean DynamicsSimulator_impl::getCharacterCollidingPairs(const char* characterName, LinkPairSequence_out pairs)
{
    return true;
}


void DynamicsSimulator_impl::calcCharacterJacobian(
    const char* characterName, const char* baseLink, const char* targetLink, DblSequence_out jacobian)
{

}

 
